From 297ab2a6fdd34c15c5eb3eb427be8daf753bcfd7 Mon Sep 17 00:00:00 2001
From: Sergey Bugaev <bugaevc@gmail.com>
Date: Tue, 20 Jul 2021 21:04:27 +0300
Subject: [PATCH] Fix a crash

During the wl_display_roundtrip() call, the popup surface could gain
focus a second time, which would cause do_set_selection() to be invoked
again. It would destroy the popup surface, which would cause the outer
do_set_selection() call to crash.

Fix this in two ways:
* Unset the callback before we perform roundtrip, to prevent any
  reentrancy.
* Check whether we (still) have a popup_surface instead of whether we
  need it; this way even if we end up without a popup surface for some
  other mysterious reason, we still won't segfault here.

Fixes https://github.com/bugaevc/wl-clipboard/issues/117
---
 src/types/copy-action.c | 8 +++++++-
 1 file changed, 7 insertions(+), 1 deletion(-)

diff --git a/src/types/copy-action.c b/src/types/copy-action.c
index 5ef9ea1..f661fc3 100644
--- a/src/types/copy-action.c
+++ b/src/types/copy-action.c
@@ -35,12 +35,18 @@ static void do_set_selection(struct copy_action *self, uint32_t serial) {
     /* Set the selection and make sure it reaches
      * the display before we do anything else,
      * such as destroying the surface or exiting.
+     * Make sure to unset the callback to prevent
+     * reentrancy issues.
      */
+    if (self->popup_surface != NULL) {
+        self->popup_surface->on_focus = NULL;
+        self->popup_surface->data = NULL;
+    }
     device_set_selection(self->device, self->source, serial, self->primary);
     wl_display_roundtrip(self->device->wl_display);
 
     /* Now, if we have used a popup surface, destroy it */
-    if (self->device->needs_popup_surface) {
+    if (self->popup_surface != NULL) {
         popup_surface_destroy(self->popup_surface);
         self->popup_surface = NULL;
     }
